@@ -32,6 +32,9 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include <libgnomevfs/gnome-vfs.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
+
 #include "bacon-video-widget.h"
 #include "totem-interface.h"
 #include "totem-mozilla-options.h"
@@ -65,12 +68,18 @@ typedef struct _TotemEmbedded {
 	GtkWidget *about;
 	int width, height;
 	const char *orig_filename;
+	const char *mimetype;
 	char *filename, *href, *target;
 	BaconVideoWidget *bvw;
 	gboolean controller_hidden;
+	gboolean hidden;
 	TotemStates state;
 	GdkCursor *cursor;
 
+	/* Open menu item */
+	GnomeVFSMimeApplication *app;
+	GtkWidget *menu_item;
+
 	/* Seek bits */
 	GtkAdjustment *seekadj;
 	GtkWidget *seek;
@@ -88,6 +97,10 @@ static void totem_embedded_init (TotemEmbedded *emb) { }
 gboolean totem_embedded_play (TotemEmbedded *emb, GError **err);
 gboolean totem_embedded_pause (TotemEmbedded *emb, GError **err);
 gboolean totem_embedded_stop (TotemEmbedded *emb, GError **err);
+gboolean totem_embedded_set_local_file (TotemEmbedded *emb, const char *url, GError **err);
+
+static void totem_embedded_set_menu (TotemEmbedded *emb, gboolean enable);
+static void on_open1_activate (GtkButton *button, TotemEmbedded *emb);
 
 #include "totem-mozilla-interface.h"
 
@@ -136,7 +149,8 @@ totem_embedded_set_state (TotemEmbedded *emb, TotemStates state)
 
 	image = glade_xml_get_widget (emb->xml, "emb_pp_button_image");
 	gtk_image_set_from_stock (GTK_IMAGE (image), id, GTK_ICON_SIZE_MENU);
-	gdk_window_set_cursor (GTK_WIDGET (emb->bvw)->window, cursor);
+	if (emb->hidden == FALSE)
+		gdk_window_set_cursor (GTK_WIDGET (emb->bvw)->window, cursor);
 
 	emb->state = state;
 }
@@ -180,11 +194,18 @@ totem_embedded_open (TotemEmbedded *emb)
 		g_free (msg);
 
 		g_error_free (err);
+		totem_embedded_set_pp_state (emb, FALSE);
 	} else {
 		totem_embedded_set_state (emb, STATE_PAUSED);
 		totem_embedded_set_pp_state (emb, TRUE);
 	}
 
+	if (retval == FALSE || strcmp ("fd://0", emb->filename) == 0) {
+		totem_embedded_set_menu (emb, FALSE);
+	} else {
+		totem_embedded_set_menu (emb, TRUE);
+	}
+
 	return retval;
 }
 
@@ -212,35 +233,74 @@ totem_embedded_stop (TotemEmbedded *emb, GError **err)
 	return TRUE;
 }
 
-static GdkWindow *
-totem_gtk_plug_get_toplevel (GtkPlug *plug)
+gboolean
+totem_embedded_set_local_file (TotemEmbedded *emb,
+			       const char *path, GError **err)
 {
-	Window root, parent, *children;
-	guint nchildren;
-	GdkNativeWindow xid;
+	g_message ("Setting the current path to %s", path);
 
-	g_return_val_if_fail (GTK_IS_PLUG (plug), NULL);
+	g_free (emb->filename);
+	emb->filename = g_filename_to_uri (path, NULL, NULL);
 
-	xid = gtk_plug_get_id (plug);
+	return TRUE;
+}
 
-	do
-	{
-		if (XQueryTree (GDK_DISPLAY (), xid, &root,
-					&parent, &children, &nchildren) == 0)
-		{
-			g_warning ("Couldn't find window manager window");
-			return None;
-		}
+static void
+totem_embedded_set_menu (TotemEmbedded *emb, gboolean enable)
+{
+	GtkWidget *menu, *item, *image;
+	GtkWidget *copy;
+	char *label;
 
-		if (root == parent) {
-			GdkWindow *toplevel;
-			toplevel = gdk_window_foreign_new (xid);
-			return toplevel;
-		}
+	copy = glade_xml_get_widget (emb->menuxml, "copy_location1");
+	gtk_widget_set_sensitive (copy, enable);
 
-		xid = parent;
+	if (emb->menu_item != NULL) {
+		gtk_widget_destroy (emb->menu_item);
+		emb->menu_item = NULL;
 	}
-	while (TRUE);
+	if (emb->app != NULL) {
+		gnome_vfs_mime_application_free (emb->app);
+		emb->app = NULL;
+	}
+
+	if (enable == FALSE)
+		return;
+
+	emb->app = gnome_vfs_mime_get_default_application_for_uri (emb->filename, emb->mimetype);
+	if (emb->app == NULL)
+		return;
+
+	/* translators: this is:
+	 * Open With ApplicationName
+	 * as in nautilus' right-click menu */
+	label = g_strdup_printf ("_Open with \"%s\"", emb->app->name);
+	item = gtk_image_menu_item_new_with_mnemonic (label);
+	g_free (label);
+	image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	g_signal_connect (G_OBJECT (item), "activate",
+			G_CALLBACK (on_open1_activate), emb);
+	gtk_widget_show (item);
+	emb->menu_item = item;
+
+	menu = glade_xml_get_widget (emb->menuxml, "menu");
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+}
+
+static void
+on_open1_activate (GtkButton *button, TotemEmbedded *emb)
+{
+	GList *l = NULL;
+
+	g_return_if_fail (emb->app != NULL);
+
+	l = g_list_prepend (l, emb->filename);
+	if (gnome_vfs_mime_application_launch (emb->app, l) == GNOME_VFS_OK) {
+		totem_embedded_stop (emb, NULL);
+	}
+
+	g_list_free (l);
 }
 
 static void
@@ -278,20 +338,8 @@ on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 	g_free (backend_version);
 	g_free (description);
 
-	if (GTK_IS_PLUG (emb->window)) {
-		GdkWindow *toplevel;
-
-		gtk_widget_realize (emb->about);
-		toplevel = totem_gtk_plug_get_toplevel (GTK_PLUG (emb->window));
-		if (toplevel != NULL) {
-			gdk_window_set_transient_for
-				(GTK_WIDGET (emb->about)->window, toplevel);
-			gdk_window_unref (toplevel);
-		}
-	} else {
-		gtk_window_set_transient_for (GTK_WINDOW (emb->about),
-				GTK_WINDOW (emb->window));
-	}
+	totem_interface_set_transient_for (GTK_WINDOW (emb->about),
+			GTK_WINDOW (emb->window));
 
 	g_object_add_weak_pointer (G_OBJECT (emb->about),
 			(gpointer *)&emb->about);
@@ -301,6 +349,20 @@ on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 	gtk_widget_show (emb->about);
 }
 
+static void
+on_copy_location1_activate (GtkButton *button, TotemEmbedded *emb)
+{
+	GtkClipboard *clip;
+
+	/* Set both the middle-click and the super-paste buffers */
+	clip = gtk_clipboard_get_for_display
+		(gdk_display_get_default(), GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text (clip, emb->filename, -1);
+	clip = gtk_clipboard_get_for_display
+		(gdk_display_get_default(), GDK_SELECTION_PRIMARY);
+	gtk_clipboard_set_text (clip, emb->filename, -1);
+}
+
 static void
 on_play_pause (GtkWidget *widget, TotemEmbedded *emb)
 {
@@ -401,6 +463,9 @@ on_eos_event (GtkWidget *bvw, TotemEmbedded *emb)
 	gtk_adjustment_set_value (emb->seekadj, 0);
 	if (strcmp (emb->filename, "fd://0") == 0) {
 		totem_embedded_set_pp_state (emb, FALSE);
+	} else if (g_str_has_prefix (emb->filename, "file://") != FALSE) {
+		bacon_video_widget_close (emb->bvw);
+		totem_embedded_open (emb);
 	}
 }
 
@@ -449,6 +514,7 @@ static void
 totem_embedded_add_children (TotemEmbedded *emb)
 {
 	GtkWidget *child, *container, *pp_button, *vbut;
+	BvwUseType type = BVW_USE_TYPE_VIDEO;
 	GError *err = NULL;
 	GConfClient *gc;
 	int volume;
@@ -461,15 +527,16 @@ totem_embedded_add_children (TotemEmbedded *emb)
 			GTK_WINDOW (emb->window));
 
 	if (emb->xml == NULL || emb->menuxml == NULL)
-	{
 		totem_embedded_exit (emb);
-	}
 
 	child = glade_xml_get_widget (emb->xml, "vbox1");
 	gtk_container_add (GTK_CONTAINER (emb->window), child);
 
+	if (emb->hidden)
+		type = BVW_USE_TYPE_AUDIO;
+
 	emb->bvw = BACON_VIDEO_WIDGET (bacon_video_widget_new
-			(-1, -1, BVW_USE_TYPE_VIDEO, &err));
+			(-1, -1, type, &err));
 
 	if (emb->bvw == NULL)
 	{
@@ -489,8 +556,10 @@ totem_embedded_add_children (TotemEmbedded *emb)
 
 	container = glade_xml_get_widget (emb->xml, "hbox4");
 	gtk_container_add (GTK_CONTAINER (container), GTK_WIDGET (emb->bvw));
-	gtk_widget_realize (GTK_WIDGET (emb->bvw));
-	gtk_widget_show (GTK_WIDGET (emb->bvw));
+	if (type == BVW_USE_TYPE_VIDEO) {
+		gtk_widget_realize (GTK_WIDGET (emb->bvw));
+		gtk_widget_show (GTK_WIDGET (emb->bvw));
+	}
 
 	emb->seek = glade_xml_get_widget (emb->xml, "time_hscale");
 	emb->seekadj = gtk_range_get_adjustment (GTK_RANGE (emb->seek));
@@ -525,6 +594,9 @@ totem_embedded_add_children (TotemEmbedded *emb)
 	child = glade_xml_get_widget (emb->menuxml, "about1");
 	g_signal_connect (G_OBJECT (child), "activate",
 			  G_CALLBACK (on_about1_activate), emb);
+	child = glade_xml_get_widget (emb->menuxml, "copy_location1");
+	g_signal_connect (G_OBJECT (child), "activate",
+			G_CALLBACK (on_copy_location1_activate), emb);
 }
 
 static void
@@ -563,7 +635,6 @@ totem_volume_create (void)
 
 int main (int argc, char **argv)
 {
-	gboolean window_given = FALSE;
 	TotemEmbedded *emb;
 	DBusGProxy *proxy;
 	DBusGConnection *conn;
@@ -573,6 +644,10 @@ int main (int argc, char **argv)
 	gchar *svcname;
 	GError *e = NULL;
 
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+
 	if (XInitThreads () == 0)
 	{
 		gtk_init (&argc, &argv);
@@ -583,6 +658,7 @@ int main (int argc, char **argv)
 	bacon_video_widget_init_backend (NULL, NULL);
 
 	gtk_init (&argc, &argv);
+	gnome_vfs_init ();
 	dbus_g_object_type_install_info (TOTEM_TYPE_EMBEDDED,
 		&dbus_glib_totem_embedded_object_info);
 	svcname = g_strdup_printf ("org.totem_%d.MozillaPluginService",
@@ -619,7 +695,6 @@ int main (int argc, char **argv)
 	/* TODO Add popt options */
 	for (i = 1; i < argc; i++) {
 		if (OPTION_IS (TOTEM_OPTION_XID)) {
-			window_given = TRUE;
 			if (i + 1 < argc) {
 				i++;
 				xid = (Window) g_ascii_strtoull (argv[i], NULL, 10);
@@ -651,6 +726,13 @@ int main (int argc, char **argv)
 				i++;
 				emb->target = g_strdup (argv[i]);
 			}
+		} else if (OPTION_IS (TOTEM_OPTION_MIMETYPE)) {
+			if (i + 1 < argc) {
+				i++;
+				emb->mimetype = (const char *) argv[i];
+			}
+		} else if (OPTION_IS (TOTEM_OPTION_HIDDEN)) {
+			emb->hidden = TRUE;
 		} else if (i + 1 == argc) {
 			emb->filename = g_strdup (argv[i]);
 		}
@@ -674,13 +756,11 @@ int main (int argc, char **argv)
 		emb->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	}
 
+	/* Hidden or not as well? */
 	totem_embedded_add_children (emb);
 	totem_embedded_create_cursor (emb);
-	if (!window_given || xid != 0)
+	if (emb->hidden == FALSE)
 		gtk_widget_show (emb->window);
-	else {
-		gtk_widget_realize (emb->window);
-	}
 
 	/* wait until we're embedded if we're to be, or shown */
 	if (xid != 0) {
