@@ -77,7 +77,6 @@ struct Totem {
 	/* Sidebar */
 	GtkWidget *sidebar;
 	gboolean sidebar_shown;
-	gboolean sidebar_ever_shown;
 
 	/* Separate Dialogs */
 	GtkWidget *properties;
@@ -151,6 +150,7 @@ struct Totem {
 	TotemRemote *remote;
 	BaconMessageConnection *conn;
 	TotemStates state;
+	gboolean cursor_shown;
 };
 
 GtkWidget *totem_volume_create (void);
